/***************************************************************
*   Classe de gestion de l'afficheur LCD RVB i2c JHD1313M1
*     - Bus de communication i2c : LCD = 0x3e / RGB = 0x62
*     - Afficheur basé sur un LCM1602
*       https://cdn-shop.adafruit.com/datasheets/TC1602A-01T.pdf
*
*   Auteur : Marc Silanus
*   date : 14/10/2016
*   rev 0.2
*
*   Réalisé à partir de : 
*     - rgb_lcd.h / rgb_lcd.cpp (Arduino library)
*     - 2013 Copyright (c) Seeed Technology Inc.  All right reserved.
*     - Author:Loovee
*     - 2013-9-18
***************************************************************/ 

#include <stdio.h>
#include <iostream>
#include "rgb_lcd.h"

using namespace std;

rgb_lcd::rgb_lcd()
{
    
    I2C_RGB = new Ci2cBcm2835(RGB_ADDRESS);
    I2C_LCD = new Ci2cBcm2835(LCD_ADDRESS); 
    
}


void rgb_lcd::i2c_send_byte(unsigned char dta)
{
    unsigned char buffer[1];
    buffer[0] = dta;
    
    cout << hex << I2C_RGB->getAddrSlave() << " : ";
    cout << "write to RGB : ";
    
    if(I2C_RGB->writeToI2C(buffer, 1))cout << "NOK";
    else cout << "OK" ;
    cout << " -> " << hex << (int)dta ;
    cout << endl;
    usleep(200);
}

void rgb_lcd::i2c_send_byteS(unsigned char *dta, unsigned char len)
{
	
    cout << hex << I2C_LCD->getAddrSlave() << " : ";
    cout << "write to LCD : ";
    
    if(I2C_LCD->writeToI2C(dta, len)) cout << "NOK";
    else cout << "OK " ;
    for(int i=0;i<len;i++) cout << " -> " << hex << (int)dta[i] ;
    cout << endl;
    usleep(200);
}



void rgb_lcd::begin(int cols, int lines, int dotsize) 
{
    
    if (lines > 1) {
        _displayfunction |= LCD_2LINE;
    }
    _numlines = lines;
    _currline = 0;

    // for some 1 line displays you can select a 10 pixel high font
    if ((dotsize != 0) && (lines == 1)) {
        _displayfunction |= LCD_5x10DOTS;
    }

    usleep(50000);

    // set # lines, font size, etc.
    command(LCD_FUNCTIONSET | _displayfunction);
    
    usleep(40);

    // turn the display on with no cursor or blinking default
    _displaycontrol = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;
    display();
    
    usleep(40);

    // clear it off
    clear();
    
    usleep(1600);

    // Initialize to default text direction (for romance languages)
    _displaymode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;
    // set the entry mode
    command(LCD_ENTRYMODESET | _displaymode);
    
    
    // backlight init
    setReg(REG_MODE1, 0);
    // set LEDs controllable by both PWM and GRPPWM registers
    setReg(REG_OUTPUT, 0xFF);
    // set MODE2 values
    // 0010 0000 -> 0x20  (DMBLNK to 1, ie blinky mode)
    setReg(REG_MODE2, 0x20);
    
    setColorWhite();

}

/********** high level commands, for the user! */
void rgb_lcd::clear()
{
    command(LCD_CLEARDISPLAY);        // clear display, set cursor position to zero
    usleep(2000);          // this command takes a long time!
}

void rgb_lcd::home()
{
    command(LCD_RETURNHOME);        // set cursor position to zero
    usleep(2000);        // this command takes a long time!
}

void rgb_lcd::setCursor(int col, int row)
{

    col = (row == 0 ? col|0x80 : col|0xc0);
    unsigned char dta[2] = {0x80, col};

    i2c_send_byteS(dta, 2);

}

// Turn the display on/off (quickly)
void rgb_lcd::noDisplay()
{
    _displaycontrol &= ~LCD_DISPLAYON;
    command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void rgb_lcd::display() {
    _displaycontrol |= LCD_DISPLAYON;
    command(LCD_DISPLAYCONTROL | _displaycontrol);
}

// Turns the underline cursor on/off
void rgb_lcd::noCursor()
{
    _displaycontrol &= ~LCD_CURSORON;
    command(LCD_DISPLAYCONTROL | _displaycontrol);
}

void rgb_lcd::cursor() {
    _displaycontrol |= LCD_CURSORON;
    command(LCD_DISPLAYCONTROL | _displaycontrol);
}

// Turn on and off the blinking cursor
void rgb_lcd::noBlink()
{
    _displaycontrol &= ~LCD_BLINKON;
    command(LCD_DISPLAYCONTROL | _displaycontrol);
}
void rgb_lcd::blink()
{
    _displaycontrol |= LCD_BLINKON;
    command(LCD_DISPLAYCONTROL | _displaycontrol);
}

// These commands scroll the display without changing the RAM
void rgb_lcd::scrollDisplayLeft(void)
{
    command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT);
}
void rgb_lcd::scrollDisplayRight(void)
{
    command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT);
}

// This is for text that flows Left to Right
void rgb_lcd::leftToRight(void)
{
    _displaymode |= LCD_ENTRYLEFT;
    command(LCD_ENTRYMODESET | _displaymode);
}

// This is for text that flows Right to Left
void rgb_lcd::rightToLeft(void)
{
    _displaymode &= ~LCD_ENTRYLEFT;
    command(LCD_ENTRYMODESET | _displaymode);
}

// This will 'right justify' text from the cursor
void rgb_lcd::autoscroll(void)
{
    _displaymode |= LCD_ENTRYSHIFTINCREMENT;
    command(LCD_ENTRYMODESET | _displaymode);
}

// This will 'left justify' text from the cursor
void rgb_lcd::noAutoscroll(void)
{
    _displaymode &= ~LCD_ENTRYSHIFTINCREMENT;
    command(LCD_ENTRYMODESET | _displaymode);
}

// Allows us to fill the first 8 CGRAM locations
// with custom characters
void rgb_lcd::createChar(int location, int charmap[])
{

    location &= 0x7; // we only have 8 locations 0-7
    command(LCD_SETCGRAMADDR | (location << 3));
    
    
    unsigned char dta[9];
    dta[0] = 0x40;
    for(int i=0; i<8; i++)
    {
        dta[i+1] = charmap[i];
    }
    i2c_send_byteS(dta, 9);
}

// Control the backlight LED blinking
void rgb_lcd::blinkLED(void)
{
    // blink period in seconds = (<reg 7> + 1) / 24
    // on/off ratio = <reg 6> / 256
    setReg(0x07, 0x17);  // blink every second
    setReg(0x06, 0x7f);  // half on, half off
}

void rgb_lcd::noBlinkLED(void)
{
    setReg(0x07, 0x00);
    setReg(0x06, 0xff);
}

/*********** mid level commands, for sending data/cmds */

// send command
inline void rgb_lcd::command(int value)
{
    unsigned char dta[2] = {0x80, value};
    i2c_send_byteS(dta, 2);
}

// send data
int rgb_lcd::write(string text)
{
    for(int i=0;i<text.length();i++)
    {
       unsigned char dta[2] = {0x40, text[i]};
       i2c_send_byteS(dta, 2);
    }
    return 1; // assume sucess
}

void rgb_lcd::setReg(unsigned char addr, unsigned char dta)
{
	cout << hex << I2C_RGB->getAddrSlave() << " : Write to RGB : " ;
    unsigned char buffer[2];
	buffer[0] = addr;
    buffer[1] = dta;
    
    if (I2C_RGB->writeToI2C(buffer, 2)) cout << "NOK";
    else cout << "OK -> ";
    cout << hex << (int)buffer[0] << " -> " << hex << (int)buffer[1] << endl;
    usleep(200);	    
}

void rgb_lcd::setRGB(unsigned char r, unsigned char g, unsigned char b)
{
    setReg(REG_RED, r);
    setReg(REG_GREEN, g);
    setReg(REG_BLUE, b);
}

const unsigned char color_define[4][3] = 
{
    {255, 255, 255},            // white
    {255, 0, 0},                // red
    {0, 255, 0},                // green
    {0, 0, 255},                // blue
};

void rgb_lcd::setColor(unsigned char color)
{
    if(color > 3)return ;
    setRGB(color_define[color][0], color_define[color][1], color_define[color][2]);
}
